# FILE: mmDataTable.tcl
#
# Display Table of data values
#

# This code can be source'd from inside another script by setting
# no_net and no_event_loop as desired before sourcing this file.
# If no_net is set true, use the proc "AddTriples { triples }" to add
# data.

if {![info exists no_net]} {
    if {[string match "--nonet" [lindex $argv 0]]} {
	set no_net 1
    } else {
	set no_net 0      ;# Default is use net
    }
}

if {![info exists no_event_loop]} {
    set no_event_loop 0   ;# Default is to enter the event loop
}

# Support libraries
package require Oc
package require Tk
package require Ow
if {!$no_net} { package require Net }

Oc_Main SetAppName mmDataTable
Oc_Main SetVersion 1.1.0.0
regexp \\\044Date:(.*)\\\044 {$Date: 1999/03/30 21:21:48 $} _ date
Oc_Main SetDate [string trim $date]
# regexp \\\044Author:(.*)\\\044 {$Author: dgp $} _ author
# Oc_Main SetAuthor [Oc_Person Lookup [string trim $author]]
Oc_Main SetAuthor [Oc_Person Lookup donahue]
Oc_Main SetHelpURL [Oc_Url FromFilename [file join [file dirname \
        [file dirname [file dirname [Oc_DirectPathname [info \
        script]]]]] doc userguide Data_Table_Display_mmDataTa.html]]

source [file join [file dirname [info script]] dialog.tcl]

# Death
proc Die { win } {
    if {![string match "." $win]} { return }  ;# Child destroy event
    global protocol
    if {[info exists protocol]} {
	$protocol Delete; after 2000 exit
    } else {
	exit
    }
}
bind . <Destroy> {+Die %W}

set menubar .mb
foreach {fmenu dmenu omenu hmenu} \
        [Ow_MakeMenubar . $menubar File Data Options Help] {}
$fmenu add command -label "Exit" -command { Die . } -underline 1
$dmenu configure -tearoff 1 -title "Data: [Oc_Main GetInstanceName]"
$omenu add command -label "Format..." -underline 0 \
	-command { ChangeFormatInteractive }
$omenu add command -label "Select all" -underline 0 \
        -command { ToggleSelectAll 1 }
$omenu add command -label "Select none" -underline 7 \
        -command { ToggleSelectAll 0 }
$omenu add command -label "Copy" -underline 0 \
        -command { CopySelected } -accelerator "Ctrl+C"
bind . <Control-c> CopySelected
Ow_StdHelpMenu $hmenu

set menuwidth [Ow_GuessMenuWidth $menubar]
set brace [canvas .brace -width $menuwidth -height 0 -borderwidth 0 \
        -highlightthickness 0]
pack $brace -side top

set panel [frame .panel -relief groove]
pack $panel -side top -fill x -expand 1 -anchor nw
grid columnconfigure $panel 0 -weight 0
grid columnconfigure $panel 1 -weight 0
grid columnconfigure $panel 2 -weight 1

wm title . [Oc_Main GetInstanceName]
wm iconname . [wm title .]
Ow_SetIcon .

if {$tk_version>=8.0 && [string match windows $tcl_platform(platform)]} {
    # Windows doesn't size Tcl 8.0 menubar cleanly
    Oc_DisableAutoSize .
    wm geometry . "${menuwidth}x0"
    update
    wm geometry . {}
    Oc_EnableAutoSize .
    update
}

proc GetSelectList {} {
    global select displaylist
    set selectlist {}
    foreach label $displaylist {
        if { $select($label) } {
            lappend selectlist $label
        }
    }
    return $selectlist
}

proc ToggleSelectAll { {state {}} } {
    # If 'state' is {}, toggles selection of all displayed entries,
    # Otherwise sets selection of all displayed entries as specified.
    # In all cases, also toggles selection of on all non-displayed
    # entries.
    global value onoff select
    foreach label [array names value] {
        if {$onoff($label)} {
            ToggleSelect $label $state
        } else {
            # Force off if not displayed (safety)
            set select($label) 0
        }
    }
}

proc ToggleSelect { label { state {} } } {
    # Sets select status for entry 'label'.  If 'state' is {}, then
    # toggles current state, in which case this routine *assumes*
    # that select($label) is previously initialized.
    global select panel onoff
    if {[string match {} $state]} { set state [expr !$select($label)] }
    set foo [CompressLabel $label]
    if {$state} {
        # Select
        if {[catch {
            $panel.label$foo configure \
		    -background $select(onback) -foreground $select(onfore)
            $panel.sep$foo configure \
		    -background $select(onback) -foreground $select(onfore)
            $panel.value$foo configure \
		    -background $select(onback) -foreground $select(onfore)}]
        } {
            # Error occured; Entry is probably hidden.  Override
            # request and deselect.
            set state 0
        }
    }
    if {!$state} {
        # Deselect
        catch {
            $panel.label$foo configure \
		    -background $select(offback) -foreground $select(offfore)
            $panel.sep$foo configure \
		    -background $select(offback) -foreground $select(offfore)
            $panel.value$foo configure \
		    -background $select(offback) -foreground $select(offfore)
        }
    }
    set select($label) $state
}

proc DisplayListAppend { label } {
    # Routine for adding and removing entries from display
    global panel unit onoff fmt value fmtvalue justify select displaylist
    set foo [CompressLabel $label]
    if {$onoff($label)} {
        # Display entry
	if {![winfo exists $panel.label$foo]} {
	    if {[string length $unit($label)]} {
		set txt "$label ($unit($label))"
	    } else {
		set txt "$label"
	    }
	    label $panel.label$foo -text $txt -anchor e
            bind $panel.label$foo <Button-1> [list ToggleSelect $label]
            bind $panel.label$foo <Button-3> [list QuickFormat $label]
            lappend displaylist $label
	}
	if {![winfo exists $panel.sep$foo]} {
	    label $panel.sep$foo -text ":"
            bind $panel.sep$foo <Button-1> [list ToggleSelect $label]
            bind $panel.sep$foo <Button-3> [list QuickFormat $label]
	}
	if {![winfo exists $panel.value$foo]} {
	    catch {set fmtvalue($label) [format $fmt($label) $value($label)]}
	    label $panel.value$foo -textvariable fmtvalue($label)
	    if {![info exists justify($label)]} {
		set justify($label) $justify(default)
	    }
	    if {[string match "r" $justify($label)]} {
		$panel.value$foo configure -anchor e
	    } elseif {[string match "c" $justify($label)]} {
		$panel.value$foo configure -anchor center
	    } else {
		$panel.value$foo configure -anchor w
	    }
            bind $panel.value$foo <Button-1> [list ToggleSelect $label]
            bind $panel.value$foo <Button-3> [list QuickFormat $label]
	}
        set select($label) 0
	grid $panel.label$foo $panel.sep$foo $panel.value$foo
	grid configure $panel.label$foo -sticky ew
	grid configure $panel.value$foo -sticky ew
    } else {
        # Hide entry
	if {[winfo exists $panel.label$foo] && \
		[winfo exists $panel.value$foo]} {
	    grid forget $panel.label$foo $panel.value$foo $panel.sep$foo 
	    destroy $panel.label$foo $panel.value$foo $panel.sep$foo
	}
        while {[set index [lsearch -exact $displaylist $label]]>=0} {
            set displaylist [lreplace $displaylist $index $index]
        }
    }
    wm geometry . {}   ;# Set geometry to autosize
}

proc ChangeFormat { newformat {labellist {}} } {
    # Resets default format iff labellist is an empty list
    global fmt value fmtvalue onoff
    if {[llength $labellist]<1} {
	set lbllist [array names value]
	set fmt(default) $newformat
    } else {
	set lbllist $labellist
    }
    foreach lbl $lbllist {
	set fmt($lbl) $newformat
	if {[info exists onoff($lbl)] && $onoff($lbl) && \
	    [info exists value($lbl)] } {
	    catch {set fmtvalue($lbl) [format $newformat $value($lbl)]}
	}
    }
}

proc ChangeJustify { newside {labellist {}} } {
    # Resets default justify iff labellist is an empty list
    global panel justify value fmtvalue onoff
    switch -exact -- $newside {
        r { set anchor e }
        c { set anchor center }
        l -
        default { set anchor w ; set newside "l" }
    }
    if {[llength $labellist]<1} {
	set lbllist [array names value]
	set justify(default) $newside
    } else {
	set lbllist $labellist
    }
    foreach lbl $lbllist {
	set justify($lbl) $newside
	if {[info exists onoff($lbl)] && $onoff($lbl) && \
		[info exists value($lbl)] } {
	    set foo [CompressLabel $lbl]
            $panel.value$foo configure -anchor $anchor
	}
    }
}

proc ChangeFormatInteractive { {label {}} } {
    # If import label is specified, use the current settings of that
    # label to initialize the format and justification specs.
    global fmt justify
    if {[string match {} $label] || ![info exists fmt($label)] \
	|| ![info exists justify($label)]} {
	set myfmt $fmt(default)
	set myjst $justify(default)
    } else {
	set myfmt $fmt($label)
	set myjst $justify($label)
    }
    set selectlist [GetSelectList]
    if {[string match {} $selectlist]} {
        set allowselect 0
    } else {
        set allowselect 1
    }
    set result [FormatDialog "Format -- [Oc_Main GetInstanceName]" \
	    $myfmt $myjst $allowselect]
    if {![string match {} $result]} {
        foreach {myfmt myjst mysel}  $result {}
        if {[string match "all" $mysel]} {
	    set selectlist {}
	}
        ChangeFormat $myfmt $selectlist
        ChangeJustify $myjst $selectlist
	ToggleSelectAll 0
    }
}

proc QuickFormat { label } {
    ToggleSelect $label 1
    ChangeFormatInteractive $label
}

proc dtSelectionHandler { str offset maxbytes } {
    return [string range $str $offset [expr $offset+$maxbytes]]
}

proc CopySelected {} {
    # Copies "selected" entries into both the PRIMARY and
    # CLIPBOARD selections.
    global panel fmtvalue justify

    # Collect display labels and values, and determine lengths
    # (to support column alignment).
    set lbllen 0
    set vallen 0
    set dsplist {}
    foreach label [GetSelectList] {
	set foo [CompressLabel $label]
	set lbl \
		"[$panel.label$foo cget -text] [$panel.sep$foo cget -text] "
	set len [string length $lbl]
	if {$len>$lbllen} { set lbllen $len }
	lappend dsplist $lbl

	set val $fmtvalue($label)
	set len [string length $val]
	if {$len>$vallen} { set vallen $len }
	lappend dsplist $val

	lappend dsplist $justify($label)
    }

    # Construct display string, with embedded newlines
    set str {}
    foreach { lbl val jst } $dsplist {
	append str [format "%*s" $lbllen $lbl]
	switch -exact -- $jst {
	    r {
		append str [format "%*s\n" $vallen $val]
	    }
	    c {
		set len [expr ($vallen + [string length $val])/2]
		append str [format "%*s\n" $len $val]
	    }
	    l -
	    default: {
		append str [format "%s\n" $val]
	    }
	}
    }

    # Copy to clipboard
    clipboard clear
    clipboard append -- $str

    # Make available as primary selection
    selection handle . [list dtSelectionHandler $str]
    selection own .
}

######################################################################
# PROC: AddTriples
#
# Data import routine for mmDataTable widget
# Parameters:
#  Each input consists of a triple of values: label, unit, value
# Data storage:
#  The first element of each import triple is used as an index into
# the following collection of global arrays:
#
#     unit($label)     :  Value units
#     fmt($label)      :  Format code to use for display
#     justify($label)  :  Value justification: one of l, c, or r
#     value($label)    :  Value
#     fmtvalue($label) :  Formatted value
#     onoff($label)    :  Whether or not value is currently displayed
#     select($label)   :  Whether or not value is currently selected
#     displaylist      :  List of displayed entries, in display order
#
#  The fmt and justify arrays also include a "default" element which is
# applied to new labels.  The select array includes elts "onfore/back" and
# "offfore/back" which are the selected & deselected fore/background colors.
#
global unit fmt value fmtvalue onoff select displaylist
set fmt(default) "%g"    ;# Default format
set justify(default) "l" ;# Default justification
label .dummylabel
set select(offback) [.dummylabel cget -background]
set select(offfore) [.dummylabel cget -foreground]
destroy .dummylabel
text .dummytext
set select(onback)  [.dummytext cget -selectbackground]
set select(onfore)  [.dummytext cget -selectforeground]
destroy .dummytext
set displaylist {}

proc CompressLabel { lbl } {
    regsub -all "\[ \r\t\n\]" $lbl {} foo
    return $foo
}
proc AddTriples { triples } {
    global unit fmt value fmtvalue dmenu onoff select
    foreach trip $triples {
        set lbl [lindex $trip 0]
	set value($lbl) [lindex $trip 2]
        if {![info exists unit($lbl)]} {
            set unit($lbl) [lindex $trip 1]
            set onoff($lbl) 0
	    set fmt($lbl) $fmt(default)
            set select($lbl) 0
            regsub -all "\[\r\n\t\]" $lbl " " txt
            $dmenu add check -label $txt \
                    -variable onoff($lbl) \
                    -command [list DisplayListAppend $lbl]
        }
	if {$onoff($lbl)} {
	    catch {set fmtvalue($lbl) [format $fmt($lbl) $value($lbl)]}
	}
    }
}

if {!$no_net} {
    Net_Protocol New protocol -name "OOMMF DataTable protocol 0.0"
    $protocol AddMessage start HasGui {} {
	return [list start [list 0 0]]
    }
    $protocol AddMessage start DataTable { triples } {
	AddTriples $triples
	return [list start [list 0 0]]
    }
    Net_Server New server -protocol $protocol -alias [Oc_Main GetAppName] \
            -log Oc_Nop
    $server Start 0
}

if {!$no_event_loop} { vwait forever }
