/* FILE: bucket.h                    -*-Mode: c++-*-
 *
 * Class for sum accumulation with improved precision
 * 
 * NOTICE: Please see the file ../../LICENSE
 *
 * Last modified on: $Date: 1999/04/26 06:25:55 $
 * Last modified by: $Author: donahue $
 */

#ifndef _NB_BUCKET
#define _NB_BUCKET

#include "oc.h"
#include "list.h"

/* End includes */     /* Optional directive to build.tcl */

//////////////////////////////////////////////////////////////////////////
// Bucket class
//   This is a helper class for BucketAccum.  Should probably be hidden.
class Nb_Bucket
{
private:
  static const ClassDoc class_doc;
public:
  int count;
  REALWIDE sum;
  void Reset() { count=0; sum=0.0; }
  Nb_Bucket() { Reset(); }
  void Add(REALWIDE amt) { sum+=amt; count++; }
};

//////////////////////////////////////////////////////////////////////////
// BucketAccum (bucket accumulate) class
//   This class provides improved precision for accumulated additions
// by grouping the additions in sets of "accumcount" at a time.  Each
// time "accumcount" items have been added together in one bucket, the
// sum is added to the next bucket up the list.
class Nb_BucketAccum
{
private:
  static const ClassDoc class_doc;
  int accumcount;
  Nb_List<Nb_Bucket> bucket;
public:
  Nb_BucketAccum(int new_accumcount=8):bucket(8)
    { accumcount=new_accumcount; }
  void Clear();
  REALWIDE Accum(REALWIDE newval);
  REALWIDE GetSum();
};

#endif // _NB_BUCKET
