# FILE: makerules.tcl
#
# This file controls the application 'pimake' by defining rules which
# describe how to build the applications and/or extensions produced by
# the source code in this directory.
#
# Verify that this script is being sourced by pimake
if {[llength [info commands MakeRule]] == 0} {
    error "'[info script]' must be evaluated by pimake"
}
#
########################################################################

set ocObjects {
	autobuf
	byte
	imports
	messages
	oc
}

MakeRule Define {
    -targets            all
    -dependencies       [concat configure [Platform StaticLibrary oc]]
}

MakeRule Define {
    -targets            configure
    -dependencies       [Platform Specific port.h]
}

MakeRule Define {
    -targets		[Platform Specific port.h]
    -dependencies	[concat procs.tcl [Platform Executables varinfo]]
    -script		{Oc_MakePortHeader [Platform Executables varinfo] \
			        [Platform Specific port.h]
			}
}

MakeRule Define {
    -targets		[Platform Executables varinfo]
    -dependencies	[Platform Objects varinfo]
    -script		{Platform Link -obj varinfo -sub CONSOLE \
       -hpux {-lm} -linalp {-lm} -lintel {-lm} -aix {-lm} -duxalp {-lm} \
			        -out varinfo
			} 
}
## Linux/AXP requires the math library (-lm) to build varinfo.  So do
## AIX and HP-UX.  It is a guess that Digital Unix does also.

MakeRule Define {
    -targets		[Platform Objects varinfo]
    -dependencies	[concat [list [Platform Name]] \
			        [[CSourceFile New _ varinfo.cc] Dependencies]]
    -script		{Platform Compile C++ -opt 1 \
			        -inc [[CSourceFile New _ varinfo.cc] DepPath] \
			        -src varinfo.cc -out varinfo
			}
}

MakeRule Define {
    -targets            [Platform Name]
    -dependencies       {}
    -script             {MakeDirectory [Platform Name]}
}

MakeRule Define {
    -targets            tclIndex
    -dependencies       [glob -nocomplain *.tcl]
    -script             {
                         MakeTclIndex .
                        }
}

MakeRule Define {
    -targets            [Platform StaticLibrary oc]
    -dependencies	[concat tclIndex [Platform Objects $ocObjects]]
    -script		[format {
			    eval file delete [Platform StaticLibrary oc]
			    Platform MakeLibrary -out oc -wintel user32.lib \
			            -winalp user32.lib -obj {%s}
			    Platform IndexLibrary oc 
			} $ocObjects]
}

MakeRule Define {
    -targets		[Platform Objects autobuf]
    -dependencies	[concat [list [Platform Name]] \
			        [[CSourceFile New _ autobuf.cc] Dependencies]]
    -script		{Platform Compile C++ -opt 1 \
			        -inc [[CSourceFile New _ autobuf.cc] DepPath] \
			        -src autobuf.cc -out autobuf
			}
}

# The object-building rules are uglier than I would like because of a
# "chicken and egg" problem with port.h .
# When port.h doesn't exist, the $include dependency search checks
# to see if there is a MakeRule for it.  To do that it must source
# this file.  That causes the MakeRule immediately above to be
# evaluated, and dependencies of autobuf.cc are sought.  They include
# oc.h, which is what leads us to port.h in the first place.  
# The Dependencies check regards this as a circular dependency and fails.
MakeRule Define {
    -targets		[Platform Objects oc]
    -dependencies	[concat [list [Platform Name]] oc.cc \
			        oc.h [Platform Specific port.h] \
				messages.h version.h autobuf.h \
			        [[CSourceFile New _ imports.h] Dependencies]]
    -script		{Platform Compile C++ -opt 1 \
			        -inc [[CSourceFile New _ oc.cc] DepPath] \
			        -src oc.cc -out oc 
			}
}

MakeRule Define {
    -targets		[Platform Objects byte]
    -dependencies	[concat [list [Platform Name]] byte.cc \
			        [Platform Specific port.h] byte.h]
    -script		{Platform Compile C++ -opt 1 \
			        -inc [[CSourceFile New _ byte.cc] DepPath] \
			        -src byte.cc -out byte 
			}
}

MakeRule Define {
    -targets		[Platform Objects imports]
    -dependencies	[concat [list [Platform Name]] imports.cc \
			        oc.h version.h [Platform Specific port.h] \
			        [[CSourceFile New _ imports.h] Dependencies] \
                                autobuf.h]
    -script		{Platform Compile C++ -opt 1 \
			        -inc [[CSourceFile New _ imports.cc] DepPath] \
			        -src imports.cc -out imports
			}
}

# Doubly ugly.  port.h problem plus some compilers have broken
# optimizations we have to work around
if {[catch {
    [Oc_Config RunPlatform] GetValue \
            program_compiler_c++_property_optimization_breaks_varargs
} _] || !$_} {
MakeRule Define {
    -targets		[Platform Objects messages]
    -dependencies	[concat [list [Platform Name]] messages.cc \
			        oc.h [Platform Specific port.h] \
				messages.h version.h \
			        [[CSourceFile New _ imports.h] Dependencies]]
    -script		{Platform Compile C++ -opt 1 \
			        -inc [[CSourceFile New _ messages.cc] DepPath] \
			        -src messages.cc -out messages
			}
}
} else {
MakeRule Define {
    -targets		[Platform Objects messages]
    -dependencies	[concat [list [Platform Name]] messages.cc \
			        oc.h [Platform Specific port.h] \
				messages.h version.h \
			        [[CSourceFile New _ imports.h] Dependencies]]
    -script		{Platform Compile C++ -opt 0 \
			        -inc [[CSourceFile New _ messages.cc] DepPath] \
			        -src messages.cc -out messages
			}
}
}

MakeRule Define {
    -targets		upgrade
    -script		{DeleteFiles functions.h}
}

MakeRule Define {
    -targets            distclean
    -dependencies       clean
    -script             {DeleteFiles [Platform Specific port.h]
			 DeleteFiles [Platform Executables varinfo]
			 DeleteFiles [Platform Name]}
}
 
MakeRule Define {
    -targets            clean
    -dependencies       mostlyclean
}
 
MakeRule Define {
    -targets            mostlyclean
    -dependencies       objclean
    -script             {eval DeleteFiles [Platform StaticLibrary oc]}
}
 
MakeRule Define {
    -targets            objclean
    -dependencies       {}
    -script             [format {
			    eval DeleteFiles [Platform Objects {varinfo %s}]
			} $ocObjects]
}

unset ocObjects
